/*
   This program should give you a starting point for assignment 7.
*/
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "glut.h"
#include "glui.h"

/* Prototypes for the GLUT callback functions */
void display(void);
void reshape(int, int);
void slider1(double);
void slider2(double);
void keyboard(unsigned char, int, int);
void idle(void);
void timer(int);

int window;
int windowHeight; // this is set by the glut reshape() callback function
int windowWidth;  // this is set by the glut reshape() callback function

#define sliderInit 0.0
double angle1 = 0.0, angle2 = 0.0;

#define PI 3.14159265358979
double amp = 0.4;
int textureOn = 1;
int runAnimation = 1;
double deltaTime = 0.1;
double t = 0.0;  // t for "time"

/*  Create a texture  */
#define rawImageWidth  1024  // these dimensions are for the earth.raw image
#define rawImageHeight 512
GLubyte rawImage[rawImageHeight][rawImageWidth][3];

GLuint texName;

/*  Read a texture from a "raw" image file  */
void readRawImage(void)
{
   FILE* file;

   file = fopen("earth.raw", "rb");
   if (!file)
   {
      printf("Unable to open file: %s\n", "earth.raw");
      exit(1);
   }
   // Now read the raw image file into the array rawImage
   fread(rawImage, rawImageHeight * rawImageWidth * 3, 1, file);
   fclose(file);
}


int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(1000, 600);
   glutInitWindowPosition(200, 200);
   window = glutCreateWindow(argv[0]);

   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);
   //glutIdleFunc(idle);
   glutTimerFunc(20, timer, 0); // timer() is called in 20 milliseconds (50 times a second)

   // parent window, x, y, width, height, initial value, callback function
   gluiHorizontalSlider(window, 1,  0, -1, 20, sliderInit, slider1);  // top edge
   gluiVerticalSlider  (window, 0, 20, 20, -1, sliderInit, slider2);  // left edge

   glClearColor (0.0, 0.0, 0.0, 0.0);
   glEnable(GL_DEPTH_TEST);

   /* Prepare a texture */
   readRawImage();
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
   glGenTextures(1, &texName);
   glBindTexture(GL_TEXTURE_2D, texName);

   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,     GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,     GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, rawImageWidth, rawImageHeight,
                               0, GL_RGB, GL_UNSIGNED_BYTE, rawImage);

   printf("\nCS 45500 Homework Assignment 7 baby demo:\n");
   printf("Top slider rotates around the y-axis.\n");
   printf("Vertical slider rotates around the x-axis.\n");
   printf("Use 't' and 'T' keys to turn off and on the texture.\n");
   printf("Use 's' and 'S' keys to stop and start the animation.\n");
   printf("Use 'a' and 'A' keys to increase and decrease the animation ampliture.\n");
   printf("Use 'f' and 'F' keys to make the animation faster and slower.\n");
   printf("\n");

   glutMainLoop();
   return 0;
}


void display(void)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   if (textureOn)
   {
      glEnable(GL_TEXTURE_2D);
      glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
      glBindTexture(GL_TEXTURE_2D, texName);
   }
   else  // draw a wireframe
   {
      glPolygonMode(GL_FRONT, GL_LINE);
      glPolygonMode(GL_BACK,  GL_LINE);
   }

   glViewport(0, 0, (GLsizei)windowWidth, (GLsizei)windowHeight); // left, bottom, width, height

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum(-2.0, 2.0, -1.0, 1.0, 1.0, 10.0);  // left, right, bottom, top, near, far
   //glOrtho(-2.0, 2.0, -1.0, 1.0, 1.0, 10.0);  // left, right, bottom, top, near, far

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   gluLookAt(0.0, 0.0, 2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

   glRotatef( angle1, 0.0, 1.0, 0.0);
   glRotatef( angle2, 1.0, 0.0, 0.0);

   glBegin(GL_QUAD_STRIP);
     glTexCoord2f(0.0,  1.0); glVertex3f(-2.0,  1.0, amp*sin(t-PI));
     glTexCoord2f(0.0,  0.5); glVertex3f(-2.0,  0.0, amp*sin(t-PI));
     glTexCoord2f(0.25, 1.0); glVertex3f(-1.0,  1.0, amp*sin(t-PI/2.0));
     glTexCoord2f(0.25, 0.5); glVertex3f(-1.0,  0.0, amp*sin(t-PI/2.0));
     glTexCoord2f(0.50, 1.0); glVertex3f( 0.0,  1.0, amp*sin(t));
     glTexCoord2f(0.50, 0.5); glVertex3f( 0.0,  0.0, amp*sin(t));
     glTexCoord2f(0.75, 1.0); glVertex3f( 1.0,  1.0, amp*sin(t+PI/2.0));
     glTexCoord2f(0.75, 0.5); glVertex3f( 1.0,  0.0, amp*sin(t+PI/2.0));
     glTexCoord2f(1.0,  1.0); glVertex3f( 2.0,  1.0, amp*sin(t+PI));
     glTexCoord2f(1.0,  0.5); glVertex3f( 2.0,  0.0, amp*sin(t+PI));
   glEnd();

   glBegin(GL_QUAD_STRIP);
     glTexCoord2f(0.0, 0.5);  glVertex3f(-2.0,  0.0, amp*sin(t-PI));
     glTexCoord2f(0.0, 0.0);  glVertex3f(-2.0, -1.0, amp*sin(t-PI));
     glTexCoord2f(0.25, 0.5); glVertex3f(-1.0,  0.0, amp*sin(t-PI/2.0));
     glTexCoord2f(0.25, 0.0); glVertex3f(-1.0, -1.0, amp*sin(t-PI/2.0));
     glTexCoord2f(0.50, 0.5); glVertex3f( 0.0,  0.0, amp*sin(t));
     glTexCoord2f(0.50, 0.0); glVertex3f( 0.0, -1.0, amp*sin(t));
     glTexCoord2f(0.75, 0.5); glVertex3f( 1.0,  0.0, amp*sin(t+PI/2.0));
     glTexCoord2f(0.75, 0.0); glVertex3f( 1.0, -1.0, amp*sin(t+PI/2.0));
     glTexCoord2f(1.0, 0.5);  glVertex3f( 2.0,  0.0, amp*sin(t+PI));
     glTexCoord2f(1.0, 0.0);  glVertex3f( 2.0, -1.0, amp*sin(t+PI));
   glEnd();
   glFlush();

   if (textureOn)
   {
      glDisable(GL_TEXTURE_2D);
   }
   else
   {
      glPolygonMode(GL_FRONT, GL_FILL);
      glPolygonMode(GL_BACK,  GL_FILL);
   }

   glutSwapBuffers();
}


void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   gluiReshape(width, height);
}


void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
      case 't':
         textureOn = 0;
         break;
      case 'T':
         textureOn = 1;
         break;
      case 's':
         runAnimation = 0;
         break;
      case 'S':
         runAnimation = 1;
         break;
      case 'a':
         amp += 0.1;
         printf("The amplitude = %f\n", amp);
         break;
      case 'A':
         amp -= 0.1;
         printf("The amplitude = %f\n", amp);
         break;
      case 'f':
         deltaTime += 0.01;
         printf("The \"time delta\" = %f\n", deltaTime);
         break;
      case 'F':
         deltaTime -= 0.01;
         printf("The \"time delta\" = %f\n", deltaTime);
         break;
      case 27:
         exit(0);
         break;
      default:
         break;
   }
}


void slider1(double value)
{
   angle1 = 360*value;

   glutPostRedisplay();
}


void slider2(double value)
{
   angle2 = 360*value;

   glutPostRedisplay();
}


void idle()
{


   glutSetWindow(window);
   glutPostRedisplay();
}


void timer(int value)
{
   if (runAnimation)
   {
      t += deltaTime;
      if ( t > 2.0*PI ) t = 0;
   }
   glutTimerFunc(20, timer, 0); // timer() is called in 20 milliseconds (50 times a second)

   glutSetWindow(window);
   glutPostRedisplay();
}
